@@ -41,6 +41,7 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #define OBDCONF_LOGFILE "log_file"
 #define OBDCONF_SAMPLERATE "samplerate"
 #define OBDCONF_BAUDRATE "baudrate"
+#define OBDCONF_BAUDRATEUPGRADE "baudrate_upgrade"
 ///@}
 
 /// Get "a" valid home dir in which to store a dotfile
@@ -78,6 +79,15 @@ static int obd_parseConfig(FILE *f, struct OBDGPSConfig *c, int verbose) {
 		char singleval_s[1024];
 		int singleval_i;
 		long singleval_l;
+
+		char *firstnonspace = line;
+		while(*firstnonspace != '\0' &&
+			(' ' == *firstnonspace || '\t' == *firstnonspace)) firstnonspace++;
+		if('#' == *firstnonspace) {
+			if(verbose) printf("Conf found comment: %s\n", line);
+			continue;
+		}
+
 		if(1 == sscanf(line, OBDCONF_OBDDEVICE "=%1023s", singleval_s)) {
 			if(NULL != c->obd_device) {
 				free((void *)c->obd_device);
@@ -110,6 +120,10 @@ static int obd_parseConfig(FILE *f, struct OBDGPSConfig *c, int verbose) {
 			c->baudrate = singleval_l;
 			if(verbose) printf("Conf Found baudrate: %li\n", singleval_l);
 		}
+		if(1 == sscanf(line, OBDCONF_BAUDRATEUPGRADE "=%li", &singleval_l)) {
+			c->baudrate_upgrade = singleval_l;
+			if(verbose) printf("Conf Found baudrate upgrade: %li\n", singleval_l);
+		}
 		if(1 == sscanf(line, OBDCONF_SAMPLERATE "=%i", &singleval_i)) {
 			c->samplerate = singleval_i;
 			if(verbose) printf("Conf Found samplerate: %i\n", singleval_i);
@@ -132,6 +146,7 @@ struct OBDGPSConfig *obd_loadConfig(int verbose) {
 	c->samplerate = 1;
 	c->optimisations = 0;
 	c->baudrate = -1;
+	c->baudrate_upgrade = -1;
 
 	char fullfilename[MAX_PATH];
 
@@ -198,6 +213,32 @@ struct OBDGPSConfig *obd_loadConfig(int verbose) {
 		}
 	}
 
+	// If the OBD_CONFIGFILE env var is set
+
+	char *envfile = getenv("OBD_CONFIGFILE");
+	if(verbose) {
+		printf("OBD_CONFIGFILE env var %s: %s\n",
+						NULL==envfile?"not found":"found",
+						NULL==envfile?"":envfile);
+	}
+	if(NULL != envfile) {
+		if(verbose) {
+			printf("Attempting to read %s ..", envfile);
+		}
+		f = fopen(envfile, "r");
+		if(NULL != f) {
+			if(verbose) {
+				printf("Opened. Parsing\n");
+			}
+			obd_parseConfig(f,c,verbose);
+			fclose(f);
+		} else {
+			if(verbose) {
+				printf("Couldn't open\n");
+			}
+		}
+	}
+
 	if(verbose) {
 		printf("Full Config:\n"
 					 "	" OBDCONF_OBDDEVICE ":%s\n"
@@ -206,10 +247,11 @@ struct OBDGPSConfig *obd_loadConfig(int verbose) {
 					 "	" OBDCONF_OPTIMISATIONS ":%i\n"
 					 "	" OBDCONF_SAMPLERATE ":%i\n"
 					 "	" OBDCONF_BAUDRATE ":%li\n"
+					 "	" OBDCONF_BAUDRATEUPGRADE ":%li\n"
 					 "	" OBDCONF_LOGFILE ":%s\n",
 					 	c->obd_device, c->gps_device, c->log_columns,
 						c->optimisations, c->samplerate, c->baudrate,
-						c->log_file);
+						c->baudrate_upgrade, c->log_file);
 	}
 	return c;
 }
@@ -234,6 +276,7 @@ int obd_writeConfig(struct OBDGPSConfig *c) {
 	fprintf(f, OBDCONF_OPTIMISATIONS "=%i\n", c->optimisations);
 	fprintf(f, OBDCONF_SAMPLERATE "=%i\n", c->samplerate);
 	fprintf(f, OBDCONF_BAUDRATE "=%li\n", c->baudrate);
+	fprintf(f, OBDCONF_BAUDRATEUPGRADE "=%li\n", c->baudrate_upgrade);
 
 	fclose(f);
 
@@ -262,8 +305,16 @@ int obd_configCmds(const char *log_columns, struct obdservicecmd ***cmds) {
 	char *currcmd = strtok(cmdlist, toklist);
 	while(currcmd) {
 		struct obdservicecmd *c;
+
+		unsigned int cmdpid;
 		if(NULL != (c = obdGetCmdForColumn(currcmd))) {
 			cols++;
+		} else if(1 == sscanf(currcmd, "%2X", &cmdpid)) {
+			if(NULL != (c = obdGetCmdForPID(cmdpid))) {
+				cols++;
+			} else {
+				printf("Warning: Couldn't find column for PID '%s'. Possible config file problem\n", currcmd);
+			}
 		} else {
 			printf("Warning: Couldn't find column '%s'. Possible config file problem\n", currcmd);
 		}
@@ -281,8 +332,14 @@ int obd_configCmds(const char *log_columns, struct obdservicecmd ***cmds) {
 	currcmd = strtok(cmdlist, toklist);
 	while(currcmd) {
 		struct obdservicecmd *c;
+
+		unsigned int cmdpid;
 		if(NULL != (c = obdGetCmdForColumn(currcmd))) {
 			(*cmds)[currcol++] = c;
+		} else if(1 == sscanf(currcmd, "%2X", &cmdpid)) {
+			if(NULL != (c = obdGetCmdForPID(cmdpid))) {
+				(*cmds)[currcol++] = c;
+			}
 		}
 		currcmd = strtok(NULL, toklist);
 	}
