@@ -31,7 +31,7 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "sqlite3.h"
 
-void kmlvalueheightcolor(sqlite3 *db, FILE *f, const char *name, const char *desc, const char *columnname, int height, const char *col, int numcols, int defaultvis, double start, double end) {
+void kmlvalueheightcolor(sqlite3 *db, FILE *f, const char *name, const char *desc, const char *columnname, int height, const char *col, int numcols, int defaultvis, double start, double end, int trip) {
 	int rc; // return from sqlite
 	sqlite3_stmt *stmt; // sqlite statement
 	const char *dbend; // ignored handle for sqlite
@@ -40,11 +40,13 @@ void kmlvalueheightcolor(sqlite3 *db, FILE *f, const char *name, const char *des
 
 	snprintf(select_sql,sizeof(select_sql),
 					"SELECT %i*%s/(SELECT MAX(%s) FROM obd "
-						"WHERE obd.time>%f AND obd.time<%f) "
+						"WHERE trip=%i) "
 					"AS height,gps.lat, gps.lon, %s "
 					"FROM obd INNER JOIN gps ON obd.time=gps.time "
-					"WHERE obd.time>%f AND obd.time<%f",
-					height, columnname, columnname, start, end, col, start, end);
+					"WHERE obd.trip=%i",
+					height, columnname, columnname, trip, col, trip);
+
+	// printf("select sql:\n%s\n", select_sql);
 
 	rc = sqlite3_prepare_v2(db, select_sql, -1, &stmt, &dbend);
 
@@ -61,11 +63,13 @@ void kmlvalueheightcolor(sqlite3 *db, FILE *f, const char *name, const char *des
 			char percentile_sql[2048]; // the actual sql
 			snprintf(percentile_sql, sizeof(percentile_sql),
 				"SELECT %s AS ckobd FROM obd "
-				"WHERE vss>0 AND obd.time>%f AND obd.time<%f "
+				"WHERE vss>0 AND obd.trip=%i "
 				"ORDER BY ckobd "
 				"LIMIT 1 OFFSET (SELECT %i*COUNT(*)/100 FROM obd "
-					"WHERE vss>0 AND obd.time>%f AND obd.time<%f)",
-				col, start, end, i*100/numcols, start, end);
+					"WHERE vss>0 AND obd.trip=%i)",
+				col, trip, i*100/numcols, trip);
+
+			// printf("Percentile sql:\n%s\n", percentile_sql);
 
 			sqlite3_stmt *pstmt; // the percentile statement
 			rc = sqlite3_prepare_v2(db, percentile_sql, -1, &pstmt, &dbend);
@@ -167,23 +171,23 @@ void kmlvalueheightcolor(sqlite3 *db, FILE *f, const char *name, const char *des
 		time_t startt = (time_t)floor(start);
 
 		fprintf(f, "<Placemark>\n"
-			"<name>Start (%s)</name>\n"
+			"<name>Start %i (%s)</name>\n"
 			"<Point>\n"
 			"<coordinates>\n"
 			"%f,%f,%f"
 			"</coordinates>\n"
 			"</Point>\n"
-			"</Placemark>\n", ctime(&startt),
+			"</Placemark>\n", trip, ctime(&startt),
 				firstpos[2],firstpos[1],firstpos[0]);
 
 		fprintf(f, "<Placemark>\n"
-			"<name>End (%s)</name>\n"
+			"<name>End %i (%s)</name>\n"
 			"<Point>\n"
 			"<coordinates>\n"
 			"%f,%f,%f"
 			"</coordinates>\n"
 			"</Point>\n"
-			"</Placemark>\n", ctime(&endt),
+			"</Placemark>\n", trip, ctime(&endt),
 				lastpos[2],lastpos[1],lastpos[0]);
 
 		fprintf(f,"</Document>\n");
