@@ -31,7 +31,10 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "sqlite3.h"
 
-void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, const char *columnname, int height, int defaultvis, double start, double end) {
+/// A distance greater than this is considered to be not zero
+#define EPSILONDIST 0.000001
+
+void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, const char *columnname, int height, int defaultvis, double start, double end, int trip) {
 	int rc; // return from sqlite
 	sqlite3_stmt *stmt; // sqlite statement
 	const char *dbend; // ignored handle for sqlite
@@ -42,8 +45,8 @@ void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, co
 	char normal_sql[1024];
 	
 	snprintf(normal_sql, sizeof(normal_sql),
-			"SELECT %i/(SELECT MAX(%s) FROM obd WHERE time>=%f AND time<=%f)",
-			height, columnname, start, end
+			"SELECT %i/(SELECT MAX(%s) FROM obd WHERE trip=%i)",
+			height, columnname, trip
 			);
 	rc = sqlite3_prepare_v2(db, normal_sql, -1, &normal_stmt, &dbend);
 	if(SQLITE_OK != rc) {
@@ -64,11 +67,12 @@ void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, co
 	char select_sql[2048]; // the select statement
 
 	snprintf(select_sql,sizeof(select_sql),
-					"SELECT T1.obdkmlthing AS height,T2.lat,T2.lon "
-					"FROM (SELECT %s AS obdkmlthing,time FROM obd WHERE time>=%f AND time<=%f) AS T1 "
-					"INNER JOIN (SELECT lat,lon,time FROM gps WHERE time>=%f and time<=%f) AS T2 "
-					"ON T1.time=T2.time ",
-					columnname, start, end, start, end);
+					"SELECT T1.obdkmlthing AS height,gps.lat,gps.lon "
+					"FROM (SELECT %s AS obdkmlthing,time FROM obd WHERE trip=%i) AS T1 "
+					"INNER JOIN gps "
+					"ON T1.time=gps.time "
+					"WHERE gps.trip=%i ",
+					columnname, trip, trip);
 
 	rc = sqlite3_prepare_v2(db, select_sql, -1, &stmt, &dbend);
 
@@ -101,7 +105,15 @@ void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, co
 		double firstpos[3] = {0,0,0};
 		double lastpos[3] = {0,0,0};
 
+		// Number of rows, total
+		long rowcount = 0;
+		// Number of rows output to kml
+		long outputcount = 0;
+
+		double totalheight = 0;
 		while(SQLITE_ROW == sqlite3_step(stmt)) {
+			rowcount++;
+
 			if(0 == have_firstpos) {
 				firstpos[2] = sqlite3_column_double(stmt, 2);
 				firstpos[1] = sqlite3_column_double(stmt, 1);
@@ -109,23 +121,35 @@ void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, co
 				have_firstpos = 1;
 			}
 
-			float height = sqlite3_column_double(stmt, 0);
-			if(abs(height) > 0.001) {
+			double currpos[3];
+			currpos[2] = sqlite3_column_double(stmt, 2);
+			currpos[1] = sqlite3_column_double(stmt, 1);
+			currpos[0] = sqlite3_column_double(stmt, 0);
+
+			float delta = sqrt((currpos[2] - lastpos[2]) * (currpos[2] - lastpos[2]) +
+					(currpos[1] - lastpos[1]) * (currpos[1] - lastpos[1]));
+			float height = normalfactor * sqlite3_column_double(stmt, 0);
+			if(delta > EPSILONDIST) {
 				ismoving = 1;
 			}
 			if(ismoving) {
-				fprintf(f, "%f,%f,%f\n", sqlite3_column_double(stmt, 2),sqlite3_column_double(stmt, 1),normalfactor*height);
+				outputcount++;
+				fprintf(f, "%f,%f,%f\n", currpos[2],currpos[1],height);
+				totalheight += height;
 			}
-			if(abs(height) < 0.001) {
+			if(delta < EPSILONDIST) {
 				ismoving = 0;
 			}
 
-			lastpos[2] = sqlite3_column_double(stmt, 2);
-			lastpos[1] = sqlite3_column_double(stmt, 1);
-			lastpos[0] = sqlite3_column_double(stmt, 0);
+			lastpos[2] = currpos[2];
+			lastpos[1] = currpos[1];
+			lastpos[0] = currpos[0];
 
 		}
 
+		printf("Total db rows: %li. KML rows: %li. Ignored rows: %li %s\n", rowcount, outputcount,
+						rowcount - outputcount,
+						outputcount<(rowcount-outputcount)?"\nOutput rows seems low":"");
 		fprintf(f,"</coordinates>\n"
 			"</LineString>\n"
 			"</Placemark>\n");
@@ -135,23 +159,23 @@ void kmlvalueheight(sqlite3 *db, FILE *f, const char *name, const char *desc, co
 		time_t startt = (time_t)floor(start);
 
 		fprintf(f, "<Placemark>\n"
-			"<name>Start (%s)</name>\n"
+			"<name>Start %i (%s)</name>\n"
 			"<Point>\n"
 			"<coordinates>\n"
 			"%f,%f,%f"
 			"</coordinates>\n"
 			"</Point>\n"
-			"</Placemark>\n", ctime(&startt),
+			"</Placemark>\n", trip, ctime(&startt),
 				firstpos[2],firstpos[1],firstpos[0]);
 
 		fprintf(f, "<Placemark>\n"
-			"<name>End (%s)</name>\n"
+			"<name>End %i (%s)</name>\n"
 			"<Point>\n"
 			"<coordinates>\n"
 			"%f,%f,%f"
 			"</coordinates>\n"
 			"</Point>\n"
-			"</Placemark>\n", ctime(&endt),
+			"</Placemark>\n", trip, ctime(&endt),
 				lastpos[2],lastpos[1],lastpos[0]);
 
 		fprintf(f,"</Document>\n");
