@@ -28,7 +28,7 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #include "sqlite3.h"
 
 int creategpstable(sqlite3 *db) {
-	char create_sql[] = "CREATE TABLE gps (lat REAL, lon REAL, alt REAL, time REAL)";
+	char create_sql[] = "CREATE TABLE IF NOT EXISTS gps (lat REAL, lon REAL, alt REAL, speed REAL, course REAL, gpstime REAL, time REAL, trip INTEGER)";
 
 	/// sqlite3 return status
 	int rc;
@@ -43,19 +43,26 @@ int creategpstable(sqlite3 *db) {
 
 
 	// Create the index
-	char create_idx_sql[] = "CREATE INDEX IDX_GPSTIME ON gps (time)";
+	char create_idx_sql[] = "CREATE INDEX IF NOT EXISTS IDX_GPSTIME ON gps (time)";
 
 	if(SQLITE_OK != (rc = sqlite3_exec(db, create_idx_sql, NULL, NULL, &errmsg))) {
 		fprintf(stderr, "Not Fatal: sqlite error creating index %s: %s\n", create_idx_sql, errmsg);
 		sqlite3_free(errmsg);
 	}
 
+	char create_idxtrip_sql[] = "CREATE INDEX IF NOT EXISTS IDX_GPSTRIP ON gps (trip)";
+
+	if(SQLITE_OK != (rc = sqlite3_exec(db, create_idxtrip_sql, NULL, NULL, &errmsg))) {
+		fprintf(stderr, "Not Fatal: sqlite error creating index %s: %s\n", create_idxtrip_sql, errmsg);
+		sqlite3_free(errmsg);
+	}
+
 
 	return 0;
 }
 
 int creategpsinsertstmt(sqlite3 *db, sqlite3_stmt **ret_stmt) {
-	char insert_sql[] = "INSERT INTO gps (lat,lon,alt,time) VALUES (?,?,?,?)";
+	char insert_sql[] = "INSERT INTO gps (lat,lon,alt,speed,course,gpstime,time,trip) VALUES (?,?,?,?,?,?,?,?)";
 
 	int rc;
 	const char *zTail;
