@@ -31,6 +31,7 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #include "database.h"
 #include "obddb.h"
 #include "gpsdb.h"
+#include "ecudb.h"
 #include "tripdb.h"
 #include "obdserial.h"
 #include "gpscomm.h"
@@ -40,6 +41,12 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 
 #ifdef HAVE_GPSD
 #include "gps.h"
+
+/// IP address of gpsd
+#define GPSD_ADDR "127.0.0.1"
+
+/// Port of gpsd
+#define GPSD_PORT "2947"
 #endif //HAVE_GPSD
 
 #ifdef HAVE_DBUS
@@ -55,24 +62,29 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #include <getopt.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif // HAVE_SIGNAL_H
 
+/// Commit transactions once every this many seconds
+#define TRANSACTIONTIME 8
+
 /// Set when we catch a signal we want to exit on
 static int receive_exitsignal = 0;
 
 /// If we catch a signal to start the trip, set this
 static int sig_starttrip = 0;
 
-/// If we catch a signal to end the trip, set this
-static int sig_endtrip = 0;
-
-#ifdef OBD_POSIX
+#ifdef OBDPLATFORM_POSIX
 /// Daemonise. Returns 0 for success, or nonzero on failure.
 static int obddaemonise();
-#endif //OBD_POSIX
+#endif //OBDPLATFORM_POSIX
+
+/// Set up signal handling
+static void install_signalhandlers();
 
 static void catch_quitsignal(int sig) {
 	receive_exitsignal = 1;
@@ -82,10 +94,6 @@ static void catch_tripstartsignal(int sig) {
 	sig_starttrip = 1;
 }
 
-static void catch_tripendsignal(int sig) {
-	sig_endtrip = 1;
-}
-
 int main(int argc, char** argv) {
 	/// Serial port full path to open
 	char *serialport = NULL;
@@ -105,12 +113,12 @@ int main(int argc, char** argv) {
 	/// Ask to show the capabilities of the OBD device then exit
 	int showcapabilities = 0;
 
+	/// Set if the user wishes to upgrade the baudrate
+	long baudrate_upgrade = -1;
+
 	/// Time between samples, measured in microseconds
 	long frametime = 0;
 
-	/// Disable automatic trip starting and stopping
-	int disable_autotrip = 0;
-
 	/// Spam all readings to stdout
 	int spam_stdout = 0;
 
@@ -123,10 +131,10 @@ int main(int argc, char** argv) {
 	/// Serial log filename
 	char *seriallogname = NULL;
 
-#ifdef OBD_POSIX
+#ifdef OBDPLATFORM_POSIX
 	/// Daemonise
 	int daemonise = 0;
-#endif //OBD_POSIX
+#endif //OBDPLATFORM_POSIX
 
 	/// Requested baudrate
 	long requested_baud = -1;
@@ -137,6 +145,8 @@ int main(int argc, char** argv) {
 	if(NULL != obd_config) {
 		samplespersecond = obd_config->samplerate;
 		enable_optimisations = obd_config->optimisations;
+		requested_baud = obd_config->baudrate;
+		baudrate_upgrade = obd_config->baudrate_upgrade;
 	}
 
 	// Do not attempt to buffer stdout at all
@@ -160,26 +170,41 @@ int main(int argc, char** argv) {
 				}
 				serialport = strdup(optarg);
 				break;
-			case 'n':
-				disable_autotrip = 1;
-				break;
 			case 'o':
 				enable_optimisations = 1;
 				break;
 			case 't':
 				spam_stdout = 1;
 				break;
-#ifdef OBD_POSIX
+			case 'u':
+				{
+					int newout = open(optarg, O_CREAT|O_RDWR|O_APPEND,
+						S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+					if(-1 == newout) {
+						perror(optarg);
+					} else {
+						printf("Redirecting output to %s\n", optarg);
+						close(STDOUT_FILENO);
+						close(STDERR_FILENO);
+						dup2(newout, STDOUT_FILENO);
+						dup2(newout, STDERR_FILENO);
+					}
+				}
+				break;
+#ifdef OBDPLATFORM_POSIX
 			case 'm':
 				daemonise = 1;
 				break;
-#endif //OBD_POSIX
+#endif //OBDPLATFORM_POSIX
 			case 'c':
 				samplecount = atoi(optarg);
 				break;
 			case 'b':
 				requested_baud = strtol(optarg, (char **)NULL, 10);
 				break;
+			case 'B':
+				baudrate_upgrade = strtol(optarg, (char **)NULL, 10);
+				break;
 			case 'd':
 				if(NULL != databasename) {
 					free(databasename);
@@ -246,17 +271,12 @@ int main(int argc, char** argv) {
 		startseriallog(seriallogname);
 	}
 
-	if(-1 == requested_baud) {
-		// Didn't choose one on the command-line
-		requested_baud = obd_config->baudrate;
-	}
 
 	// Open the serial port.
-	int obd_serial_port = openserial(serialport, requested_baud);
+	int obd_serial_port = openserial(serialport, requested_baud, baudrate_upgrade);
 
 	if(-1 == obd_serial_port) {
-		fprintf(stderr, "Couldn't open obd serial port. Exiting.\n");
-		exit(1);
+		fprintf(stderr, "Couldn't open obd serial port. Attempting to continue.\n");
 	} else {
 		fprintf(stderr, "Successfully connected to serial port. Will log obd data\n");
 	}
@@ -289,16 +309,24 @@ int main(int argc, char** argv) {
 #ifdef HAVE_GPSD
 	// Open the gps device
 	struct gps_data_t *gpsdata;
-	gpsdata = opengps("127.0.0.1", "2947");
+	gpsdata = opengps(GPSD_ADDR, GPSD_PORT);
 
 	if(NULL == gpsdata) {
-		fprintf(stderr, "Couldn't open gps port.\n");
+		fprintf(stderr, "Couldn't open gps port on startup.\n");
 	} else {
 		fprintf(stderr, "Successfully connected to gpsd. Will log gps data\n");
 	}
 
 #endif //HAVE_GPSD
 
+	if(-1 == obd_serial_port
+#ifdef HAVE_GPSD
+		&& NULL == gpsdata
+#endif //HAVE_GPSD
+	) {
+		fprintf(stderr, "Couldn't find either gps or obd to log. Exiting.\n");
+		exit(1);
+	}
 
 #ifdef HAVE_DBUS
 	obdinitialisedbus();
@@ -323,13 +351,13 @@ int main(int argc, char** argv) {
 	}
 
 	// Disable sqlite's synchronous pragma.
-	char *zErrMsg;
+	/* char *zErrMsg;
 	rc = sqlite3_exec(db, "PRAGMA synchronous=OFF",
 					NULL, NULL, &zErrMsg);
 	if(rc != SQLITE_OK) {
 		printf("SQLite error %i: %s\n", rc, zErrMsg);
 		sqlite3_free(zErrMsg);
-	}
+	} */
 
 	// Wishlist of commands from config file
 	struct obdservicecmd **wishlist_cmds = NULL;
@@ -351,12 +379,14 @@ int main(int argc, char** argv) {
 
 	createtriptable(db);
 
+	createecutable(db);
+
 	// All of these have obdnumcols-1 since the last column is time
-	int cmdlist[obdnumcols-1]; // Commands to send [index into obdcmds]
+	int cmdlist[obdnumcols-1]; // Commands to send [index into obdcmds_mode1]
 
 	int i,j;
-	for(i=0,j=0; i<sizeof(obdcmds)/sizeof(obdcmds[0]); i++) {
-		if(NULL != obdcmds[i].db_column) {
+	for(i=0,j=0; i<sizeof(obdcmds_mode1)/sizeof(obdcmds_mode1[0]); i++) {
+		if(NULL != obdcmds_mode1[i].db_column) {
 			if(isobdcapabilitysupported(obdcaps,i)) {
 				cmdlist[j] = i;
 				j++;
@@ -382,7 +412,7 @@ int main(int argc, char** argv) {
 		exit(1);
 	}
 
-#ifdef OBD_POSIX
+#ifdef OBDPLATFORM_POSIX
 	if(daemonise) {
 		if(0 != obddaemonise()) {
 			fprintf(stderr,"Couldn't daemonise, exiting\n");
@@ -390,7 +420,7 @@ int main(int argc, char** argv) {
 			exit(1);
 		}
 	}
-#endif //OBD_POSIX
+#endif //OBDPLATFORM_POSIX
 
 #ifdef HAVE_GPSD
 	// Ping a message to stdout the first time we get
@@ -398,64 +428,12 @@ int main(int argc, char** argv) {
 	int have_gps_lock = 0;
 #endif //HAVE_GPSD
 
-	// Set up signal handling
-
-#ifdef HAVE_SIGACTION
-	struct sigaction sa_new;
-
-	// Exit on ctrl+c or SIGTERM
-	sa_new.sa_handler = catch_quitsignal;
-	sigemptyset(&sa_new.sa_mask);
-	sigaddset(&sa_new.sa_mask, SIGINT);
-	sigaddset(&sa_new.sa_mask, SIGTERM);
-	sigaction(SIGINT, &sa_new, NULL);
-	sigaction(SIGTERM, &sa_new, NULL);
-
-#ifdef SIGUSR1
-	// Start a trip on USR1
-	sa_new.sa_handler = catch_tripstartsignal;
-	sigemptyset(&sa_new.sa_mask);
-	sigaddset(&sa_new.sa_mask, SIGUSR1);
-	sigaction(SIGUSR1, &sa_new, NULL);
-#endif //SIGUSR1
-
-#ifdef SIGUSR2
-	// End a trip on USR2
-	sa_new.sa_handler = catch_tripendsignal;
-	sigemptyset(&sa_new.sa_mask);
-	sigaddset(&sa_new.sa_mask, SIGUSR2);
-	sigaction(SIGUSR2, &sa_new, NULL);
-#endif //SIGUSR2
-
-#else // HAVE_SIGACTION
-
-// If your unix implementation doesn't have sigaction, we can fall
-//  back to the older [bad, unsafe] signal().
-#ifdef HAVE_SIGNAL_FUNC
-
-	// Exit on ctrl+c or TERM
-	signal(SIGINT, catch_quitsignal);
-	signal(SIGTERM, catch_quitsignal);
-
-#ifdef SIGUSR1
-	// Start a trip on USR1
-	signal(SIGUSR1, catch_tripstartsignal);
-#endif //SIGUSR1
-
-#ifdef SIGUSR2
-	// Start a trip on USR2
-	signal(SIGUSR2, catch_tripstartsignal);
-#endif //SIGUSR2
-
-#endif // HAVE_SIGNAL_FUNC
-
-
-#endif //HAVE_SIGACTION
 
+	install_signalhandlers();
 
 
 	// The current thing returned by starttrip
-	sqlite3_int64 currenttrip;
+	sqlite3_int64 currenttrip = 0;
 
 	// Set when we're actually inside a trip
 	int ontrip = 0;
@@ -463,6 +441,17 @@ int main(int argc, char** argv) {
 	// The current time we're inserting
 	double time_insert;
 
+	// The last time we tried to check the gps daemon
+	double time_lastgpscheck = 0;
+
+	// Number of samples per transaction
+	const int basetransactioncount = TRANSACTIONTIME * (0==samplespersecond?10:samplespersecond);
+
+	// Store a few seconds worth of samples per transaction
+	int transactioncount = 0;
+
+	obdbegintransaction(db);
+
 	while(samplecount == -1 || samplecount-- > 0) {
 
 		struct timeval starttime; // start time through loop
@@ -485,13 +474,6 @@ int main(int argc, char** argv) {
 						ontrip = 1;
 					}
 					break;
-				case OBD_DBUS_ENDTRIP:
-					if(ontrip) {
-						fprintf(stderr,"Ending current trip\n");
-						endtrip(db, time_insert);
-						ontrip = 0;
-					}
-					break;
 				case OBD_DBUS_NOMESSAGE:
 				default:
 					break;
@@ -501,21 +483,15 @@ int main(int argc, char** argv) {
 
 		time_insert = (double)starttime.tv_sec+(double)starttime.tv_usec/1000000.0f;
 
-		if(sig_endtrip) {
+		if(sig_starttrip) {
 			if(ontrip) {
 				fprintf(stderr,"Ending current trip\n");
-				endtrip(db, time_insert);
+				updatetrip(db, currenttrip, time_insert);
 				ontrip = 0;
 			}
-			sig_endtrip = 0;
-		}
-
-		if(sig_starttrip) {
-			if(!ontrip) {
-				currenttrip = starttrip(db, time_insert);
-				fprintf(stderr,"Created a new trip (%i)\n", (int)currenttrip);
-				ontrip = 1;
-			}
+			currenttrip = starttrip(db, time_insert);
+			fprintf(stderr,"Created a new trip (%i)\n", (int)currenttrip);
+			ontrip = 1;
 			sig_starttrip = 0;
 		}
 
@@ -525,62 +501,74 @@ int main(int argc, char** argv) {
 			// Get all the OBD data
 			for(i=0; i<obdnumcols-1; i++) {
 				float val;
-				unsigned int cmdid = obdcmds[cmdlist[i]].cmdid;
-				int numbytes = enable_optimisations?obdcmds[cmdlist[i]].bytes_returned:0;
-				OBDConvFunc conv = obdcmds[cmdlist[i]].conv;
+				unsigned int cmdid = obdcmds_mode1[cmdlist[i]].cmdid;
+				int numbytes = enable_optimisations?obdcmds_mode1[cmdlist[i]].bytes_returned:0;
+				OBDConvFunc conv = obdcmds_mode1[cmdlist[i]].conv;
 
 				obdstatus = getobdvalue(obd_serial_port, cmdid, &val, numbytes, conv);
 				if(OBD_SUCCESS == obdstatus) {
 #ifdef HAVE_DBUS
-					obddbussignalpid(&obdcmds[cmdlist[i]], val);
+					obddbussignalpid(&obdcmds_mode1[cmdlist[i]], val);
 #endif //HAVE_DBUS
 					if(spam_stdout) {
-						printf("%s=%f\n", obdcmds[cmdlist[i]].db_column, val);
+						printf("%s=%f\n", obdcmds_mode1[cmdlist[i]].db_column, val);
 					}
 					sqlite3_bind_double(obdinsert, i+1, (double)val);
-					// printf("cmd: %02X, val: %f\n",obdcmds[cmdlist[i]].cmdid,val);
+					// printf("cmd: %02X, val: %f\n",obdcmds_mode1[cmdlist[i]].cmdid,val);
 				} else {
 					break;
 				}
 			}
 
 			if(obdstatus == OBD_SUCCESS) {
+				// If they're not on a trip but the engine is going, start a trip
+				if(0 == ontrip) {
+					printf("Creating a new trip\n");
+					currenttrip = starttrip(db, time_insert);
+					ontrip = 1;
+				}
 				sqlite3_bind_double(obdinsert, i+1, time_insert);
+				sqlite3_bind_int64(obdinsert, i+2, currenttrip);
 
 				// Do the OBD insert
 				rc = sqlite3_step(obdinsert);
 				if(SQLITE_DONE != rc) {
 					printf("sqlite3 obd insert failed(%i): %s\n", rc, sqlite3_errmsg(db));
 				}
-
-				// If they're not on a trip but the engine is going, start a trip
-				if(0 == ontrip && !disable_autotrip) {
-					printf("Creating a new trip\n");
-					currenttrip = starttrip(db, time_insert);
-					ontrip = 1;
-				}
 			} else if(OBD_ERROR == obdstatus) {
 				fprintf(stderr, "Received OBD_ERROR from serial read. Exiting\n");
 				receive_exitsignal = 1;
 			} else {
 				// If they're on a trip, and the engine has desisted, stop the trip
-				if(0 != ontrip && !disable_autotrip) {
+				if(ontrip) {
 					printf("Ending current trip\n");
-					endtrip(db, time_insert);
+					updatetrip(db, currenttrip, time_insert);
 					ontrip = 0;
 				}
 			}
 			sqlite3_reset(obdinsert);
 		}
 
+		// Constantly update the trip
+		updatetrip(db, currenttrip, time_insert);
 
 #ifdef HAVE_GPSD
 		// Get the GPS data
-		double lat,lon,alt;
+		double lat,lon,alt,speed,course,gpstime;
 
 		int gpsstatus = -1;
 		if(NULL != gpsdata) {
-			gpsstatus = getgpsposition(gpsdata, &lat, &lon, &alt);
+			gpsstatus = getgpsposition(gpsdata, &lat, &lon, &alt, &speed, &course, &gpstime);
+		} else {
+			if(time_insert - time_lastgpscheck > 10) { // Try again once in a while
+				gpsdata = opengps(GPSD_ADDR, GPSD_PORT);
+				if(NULL != gpsdata) {
+					printf("Delayed connection to gps achieved\n");
+				} else {
+					// fprintf(stderr, "Delayed connection to gps failed\n");
+				}
+				time_lastgpscheck = time_insert;
+			}
 		}
 		if(gpsstatus < 0 || NULL == gpsdata) {
 			// Nothing yet
@@ -595,16 +583,22 @@ int main(int argc, char** argv) {
 			if(gpsstatus >= 1) {
 				sqlite3_bind_double(gpsinsert, 3, alt);
 			} else {
-				sqlite3_bind_double(gpsinsert, 3, -1000.0);
+				sqlite3_bind_null(gpsinsert, 3);
 			}
+			sqlite3_bind_double(gpsinsert, 4, speed);
+			sqlite3_bind_double(gpsinsert, 5, course);
+			sqlite3_bind_double(gpsinsert, 6, gpstime);
 
 			if(spam_stdout) {
-				printf("gpspos=%f,%f,%f\n", (float)lat, (float)lon, (float)(gpsstatus>=1?alt:-1000.0));
+				printf("gpspos=%f,%f,%f,%f,%f\n",
+					lat, lon, (gpsstatus>=1?alt:-1000.0), speed, course);
 			}
 
 			// Use time worked out before.
 			//  This makes table joins reliable, but the time itself may be wrong depending on gpsd lagginess
-			sqlite3_bind_double(gpsinsert, 4, time_insert);
+			sqlite3_bind_double(gpsinsert, 7, time_insert);
+
+			sqlite3_bind_int64(gpsinsert, 8, currenttrip);
 
 			// Do the GPS insert
 			rc = sqlite3_step(gpsinsert);
@@ -625,9 +619,19 @@ int main(int argc, char** argv) {
 			break;
 		}
 
-		// usleep() not as portable as select()
+
+		// Commit this if we've done more than a certain number
+		transactioncount++;
+		transactioncount%=basetransactioncount;
+		if(0 == transactioncount) {
+			obdcommittransaction(db);
+			obdbegintransaction(db);
+		}
+
 		
-		if(0 != frametime) {
+		// usleep() not as portable as select()
+
+		if(0 < frametime) {
 			selecttime.tv_sec = endtime.tv_sec - starttime.tv_sec;
 			if (selecttime.tv_sec != 0) {
 					endtime.tv_usec += 1000000*selecttime.tv_sec;
@@ -642,8 +646,10 @@ int main(int argc, char** argv) {
 		}
 	}
 
+	obdcommittransaction(db);
+
 	if(0 != ontrip) {
-		endtrip(db, time_insert);
+		updatetrip(db, currenttrip, time_insert);
 		ontrip = 0;
 	}
 
@@ -666,10 +672,11 @@ int main(int argc, char** argv) {
 	if(NULL != databasename) free(databasename);
 	if(NULL != serialport) free(serialport);
 
+	obd_freeConfig(obd_config);
 	return 0;
 }
 
-#ifdef OBD_POSIX
+#ifdef OBDPLATFORM_POSIX
 // *sniff sniff*
 // Smells like Stevens.
 static int obddaemonise() {
@@ -703,21 +710,22 @@ static int obddaemonise() {
 
 	return 0;
 }
-#endif //OBD_POSIX
+#endif //OBDPLATFORM_POSIX
 
 void printhelp(const char *argv0) {
 	printf("Usage: %s [params]\n"
 				"   [-s|--serial <" OBD_DEFAULT_SERIALPORT ">]\n"
 				"   [-c|--count <infinite>]\n"
 				"   [-i|--log-columns <" OBD_DEFAULT_COLUMNS ">]\n"
-				"   [-n|--no-autotrip]\n"
 				"   [-t|--spam-stdout]\n"
 				"   [-p|--capabilities]\n"
 				"   [-o|--enable-optimisations]\n"
-#ifdef OBD_POSIX
+				"   [-u|--output-log <filename>]\n"
+#ifdef OBDPLATFORM_POSIX
 				"   [-m|--daemonise]\n"
-#endif //OBD_POSIX
+#endif //OBDPLATFORM_POSIX
 				"   [-b|--baud <number>]\n"
+				"   [-B|--modifybaud <number>]\n"
 				"   [-l|--serial-log <filename>]\n"
 				"   [-a|--samplerate [1]]\n"
 				"   [-d|--db <" OBD_DEFAULT_DATABASE ">]\n"
@@ -729,3 +737,48 @@ void printversion() {
 }
 
 
+void install_signalhandlers() {
+	// Set up signal handling
+
+#ifdef HAVE_SIGACTION
+	struct sigaction sa_new;
+	memset(&sa_new, 0, sizeof(sa_new));
+
+	// Exit on ctrl+c or SIGTERM
+	sa_new.sa_handler = catch_quitsignal;
+	sigemptyset(&sa_new.sa_mask);
+	sigaddset(&sa_new.sa_mask, SIGINT);
+	sigaddset(&sa_new.sa_mask, SIGTERM);
+	sigaction(SIGINT, &sa_new, NULL);
+	sigaction(SIGTERM, &sa_new, NULL);
+
+#ifdef SIGUSR1
+	// Start a trip on USR1
+	sa_new.sa_handler = catch_tripstartsignal;
+	sa_new.sa_flags &= ~(SA_RESETHAND);
+	sigemptyset(&sa_new.sa_mask);
+	sigaddset(&sa_new.sa_mask, SIGUSR1);
+	sigaction(SIGUSR1, &sa_new, NULL);
+#endif //SIGUSR1
+
+#else // HAVE_SIGACTION
+
+// If your unix implementation doesn't have sigaction, we can fall
+//  back to the older [bad, unsafe] signal().
+#ifdef HAVE_SIGNAL_FUNC
+
+	// Exit on ctrl+c or TERM
+	signal(SIGINT, catch_quitsignal);
+	signal(SIGTERM, catch_quitsignal);
+
+#ifdef SIGUSR1
+	// Start a trip on USR1
+	signal(SIGUSR1, catch_tripstartsignal);
+#endif //SIGUSR1
+
+#endif // HAVE_SIGNAL_FUNC
+
+
+#endif //HAVE_SIGACTION
+
+}
