@@ -31,26 +31,27 @@ static const struct option longopts[] = {
 	{ "help", no_argument, NULL, 'h' }, ///< Print the help text
 	{ "version", no_argument, NULL, 'v' }, ///< Print the version text
 	{ "serial", required_argument, NULL, 's' }, ///< Serial Port
+	{ "db", required_argument, NULL, 'd' }, ///< Database file
 	{ "samplerate", required_argument, NULL, 'a' }, ///< Number of samples per second
 	{ "count", required_argument, NULL, 'c' }, ///< Number of values to grab
 	{ "capabilities", no_argument, NULL, 'p' }, ///< Show the capabilities the OBD device claims it can report
-	{ "no-autotrip", no_argument, NULL, 'n' }, ///< Disable automatic trip starting and stopping
 	{ "spam-stdout", no_argument, NULL, 't' }, ///< Spam readings to stdout
 	{ "serial-log", required_argument, NULL, 'l' }, ///< Log serial port data transfer
+	{ "output-log", required_argument, NULL, 'u' }, ///< Log serial port data transfer
 	{ "baud", required_argument, NULL, 'b' }, ///< Baud rate to connect at
+	{ "modifybaud", required_argument, NULL, 'B' }, ///< Upgrade to this baudrate
 	{ "log-columns", required_argument, NULL, 'i' }, ///< Log these columns
 	{ "enable-optimisations", no_argument, NULL, 'o' }, ///< Enable elm optimisations
 #ifdef OBDPLATFORM_POSIX
 	{ "daemon", no_argument, NULL, 'm' }, ///< Daemonise
 #endif //OBDPLATFORM_POSIX
-	{ "db", required_argument, NULL, 'd' }, ///< Database file
 	{ NULL, 0, NULL, 0 } ///< End
 };
 
 /// getopt() short options
-static const char shortopts[] = "htmi:b:nvs:l:c:a:op"
+static const char shortopts[] = "htd:i:b:vs:l:c:a:opu:B:"
 #ifdef OBDPLATFORM_POSIX
-	"d"
+	"m"
 #endif //OBDPLATFORM_POSIX
 ;
 
