@@ -54,11 +54,18 @@ want to call it very often. */
 struct obdservicecmd *obdGetCmdForPID(const unsigned int pid);
 
 
+// Fix "variable obdcmds defined but not used" warnings
+#ifdef __GNUC__
+#define VARIABLE_IS_NOT_USED __attribute__ ((unused))
+#else
+#define VARIABLE_IS_NOT_USED
+#endif
+
 /// List of all OBD Service commands
 /// Borrowed from various sources, mainly http://en.wikipedia.org/wiki/Table_of_OBD-II_Codes
-static struct obdservicecmd obdcmds[] = {
+static struct obdservicecmd VARIABLE_IS_NOT_USED obdcmds_mode1[] = {
 	{ 0x00, 4, NULL,            "PIDs supported 00-20" , 0, 0, "Bit Encoded", NULL, NULL },
-	{ 0x01, 4, NULL,            "Monitor status since DTCs cleared" , 0, 0, "Bit Encoded", NULL, NULL },
+	{ 0x01, 4, "dtc_cnt",            "Monitor status since DTCs cleared" , 0, 0, "Bit Encoded", NULL, NULL },
 	{ 0x02, 4, "dtcfrzf",       "DTC that caused required freeze frame data storage" , 0, 0, "Bit Encoded", NULL, NULL },
 	{ 0x03, 8, "fuelsys",       "Fuel system 1 and 2 status" , 0, 0, "Bit Encoded", NULL, NULL },
 	{ 0x04, 2, "load_pct",      "Calculated LOAD Value" , 0, 100, "%", obdConvert_04, obdRevConvert_04 },
@@ -91,7 +98,7 @@ static struct obdservicecmd obdcmds[] = {
 	{ 0x1F, 2, "runtm",         "Time Since Engine Start", 0, 65535, "seconds", obdConvert_1F, obdRevConvert_1F },
 	{ 0x20, 4, NULL,            "PIDs supported 21-40" , 0, 0, "Bit Encoded", NULL, NULL },
 	{ 0x21, 4, "mil_dist",      "Distance Travelled While MIL is Activated", 0, 65535, "km", obdConvert_21, obdRevConvert_21 },
-	{ 0x22, 2, "frp",           "Fuel Rail Pressure relative to manifold vacuum", 0, 5177.265, "kPa", obdConvert_22, obdRevConvert_22 },
+	{ 0x22, 2, "frpm",          "Fuel Rail Pressure relative to manifold vacuum", 0, 5177.265, "kPa", obdConvert_22, obdRevConvert_22 },
 	{ 0x23, 2, "frpd",          "Fuel Rail Pressure (diesel)", 0, 655350, "kPa", obdConvert_23, obdRevConvert_23 },
 	{ 0x24, 4, "lambda11",      "Bank 1 - Sensor 1/Bank 1 - Sensor 1 (wide range O2S) Oxygen Sensors Equivalence Ratio (lambda) / Voltage", 0, 2, "(ratio)", obdConvert_24_2B, obdRevConvert_24_2B },
 	{ 0x25, 4, "lambda12",      "Bank 1 - Sensor 2/Bank 1 - Sensor 2 (wide range O2S) Oxygen Sensors Equivalence Ratio (lambda) / Voltage", 0, 2, "(ratio)", obdConvert_24_2B, obdRevConvert_24_2B },
@@ -140,6 +147,32 @@ static struct obdservicecmd obdcmds[] = {
 	{ 0x50, 4, NULL,            "External Test Equipment Configuration #2" , 0, 0, "Bit Encoded", NULL, NULL },
 	{ 0x51, 2, "fuel_type",     "Fuel Type", 0, 0, "Bit Encoded", NULL, NULL },
 	{ 0x52, 2, "alch_pct",      "Ethanol fuel %", 0, 100, "%", obdConvert_52, obdRevConvert_52 },
+
+/*
+	{ 0x53, 2, "evap_vpa",      "Absolute Evap System Vapor Pressure", 0, 327.675, "kPa", NULL, NULL },
+	{ 0x54, 2, "evap_vp",       "Evap System Vapor Pressure", -32768, 32767, "Pa", NULL, NULL },
+	{ 0x55, 2, "stso2ft1",      "Short Term Secondary O2 Sensor Fuel Trim – Bank 1/3", -100, 99.22, "%", NULL, NULL },
+	{ 0x56, 2, "lgso2ft1",      "Long Term Secondary O2 Sensor Fuel Trim – Bank 1/3", -100, 99.22, "%", NULL, NULL },
+	{ 0x57, 2, "stso2ft2",      "Short Term Secondary O2 Sensor Fuel Trim – Bank 2/4", -100, 99.22, "%", NULL, NULL },
+	{ 0x58, 2, "lgso2ft2",      "Long Term Secondary O2 Sensor Fuel Trim – Bank 2/4", -100, 99.22, "%", NULL, NULL },
+	{ 0x59, 2, "frp",           "Fuel Rail Pressure (absolute)", 0, 655350, "kPa", NULL, NULL },
+	{ 0x5A, 1, "app_r",         "Relative Accellerator Pedal Position", 0, 100, "%", NULL, NULL },
+	{ 0x5B, 1, "bat_pwr",       "Hybrid Battery Pack Remaining Life", 0, 100, "%", NULL, NULL },
+	{ 0x5C, 1, "eot",           "Engine Oil Temperature", -40, 215, "Celcius", NULL, NULL },
+	{ 0x5D, 2, "fuel_timing",   "Fuel Injection Timing", -310, 301.992, "degrees", NULL, NULL },
+	{ 0x5E, 2, "fuel_rate",     "Engine Fuel Rate", 0, 3276.75, "L/h", NULL, NULL },
+	{ 0x5F, 1, "emis_sup",      "Emission requirements to which vehicle is desinged", 0, 0, "Bit Encoded", NULL, NULL },
+	{ 0x60, 4, NULL,            "PIDs supported 61-80" , 0, 0, "Bit Encoded", NULL, NULL },
+	{ 0x61, 1, "tq_dd",         "Driver's Demand Engine - Percent Torque" , -125, 130, "%", NULL, NULL },
+	{ 0x62, 1, "tq_act",        "Actual Engine - Percent Torque" , -125, 130, "%", NULL, NULL },
+	{ 0x63, 2, "tq_ref",        "Engine Reference Torque" , 0, 65535, "Nm", NULL, NULL },
+	{ 0x64, 5, "tq_max1",       "Engine Percent Torque Data" , -125, 130, "%", NULL, NULL },
+	{ 0x65, 2, NULL,            "Auxilary Inputs/Outputs" , 0, 0, "Bit Encoded", NULL, NULL },
+	{ 0x66, 5, "mafa",          "Mass Air Flow Sensor" , 0, 2047.96875, "g/s", NULL, NULL },
+	{ 0x67, 3, "ect1",          "Engine Coolant Temperature" , -40, 215, "Celcius", NULL, NULL },
+	{ 0x68, 7, "iat11",         "Intake Air Temperature Bank 1 Sensor 1" , -40, 215, "Celcius", NULL, NULL },
+	{ 0x69, 6, NULL,            "Commanded EGR and EGR Error" , 0, 0, "Bit Encoded", NULL, NULL },
+*/
 	{ 0x00, 0, NULL,            NULL, 0, 0, NULL, NULL, NULL }
 };
 
