@@ -22,6 +22,7 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <dlfcn.h>
 
 #include "simdl_datasource.h"
@@ -39,6 +40,8 @@ struct dlopen_gen {
 	int (*simdl_getvalue)(void *gen, unsigned int mode, unsigned int PID,
 	        unsigned int *A, unsigned int *B, unsigned int *C, unsigned int *D);
 	int (*simdl_idle)(void *gen, int timems);
+	int (*simdl_geterrorcodes)(void *gen, unsigned int *errorcodes, int num_codes, int *mil);
+	int (*simdl_clearerrorcodes)(void *gen);
 };
 
 const char *dlopen_simgen_name() {
@@ -47,7 +50,7 @@ const char *dlopen_simgen_name() {
 
 const char *dlopen_simgen_longdesc() {
 	return "Dynamically open a shared library plugin\n"
-		"Seed: <dynamic library filename>";
+		"Seed: <dynamic library filename[,seed for dlopen plugin]>";
 }
 
 int dlopen_simgen_create(void **gen, const char *seed) {
@@ -56,9 +59,20 @@ int dlopen_simgen_create(void **gen, const char *seed) {
 		return 1;
 	}
 
-	void *handle = dlopen(seed, RTLD_NOW);
+	char *seedcpy = strdup(seed);
+
+	char *libname = seedcpy;
+
+	char *subseed = libname;
+	for(;'\0' != *subseed && ',' != *subseed; subseed++);
+	if(',' == *subseed) {
+		*subseed = '\0';
+		subseed++;
+	};
+
+	void *handle = dlopen(libname, RTLD_NOW);
 	if(NULL == handle) {
-		fprintf(stderr, "Error dlopening %s: %s\n", seed, dlerror());
+		fprintf(stderr, "Error dlopening %s: %s\n", libname, dlerror());
 		return 1;
 	}
 
@@ -111,16 +125,27 @@ int dlopen_simgen_create(void **gen, const char *seed) {
 		fprintf(stderr, "Couldn't find symbol simdl_idle in library [Not Fatal]: %s\n", error);
 	}
 
-	printf("Successfully dlopen'd sim generator %s, %s\n", seed, g->simdl_name());
+	*(void **)(&g->simdl_geterrorcodes) = dlsym(handle, "simdl_geterrorcodes");
+	if(NULL != (error = dlerror())) {
+		fprintf(stderr, "Couldn't find symbol simdl_geterrorcodes in library [Not Fatal]: %s\n", error);
+	}
 
-	// TODO: Do something useful with this seed
-	if(1 == g->simdl_create(&(g->gen_gen), "")) {
+	*(void **)(&g->simdl_clearerrorcodes) = dlsym(handle, "simdl_clearerrorcodes");
+	if(NULL != (error = dlerror())) {
+		fprintf(stderr, "Couldn't find symbol simdl_clearerrorcodes in library [Not Fatal]: %s\n", error);
+	}
+
+	printf("Successfully dlopen'd sim generator %s, %s\n", libname, g->simdl_name());
+
+	if(1 == g->simdl_create(&(g->gen_gen), subseed)) {
 		fprintf(stderr, "dlopene'd library reported error on creation\n");
 		dlclose(handle);
 		free(g);
 		return 1;
 	}
 
+	free(seedcpy);
+
 	*gen = g;
 	return 0;
 }
@@ -145,6 +170,22 @@ int dlopen_simgen_idle(void *gen, int idlems) {
 	return 0;
 }
 
+int dlopen_simgen_geterrorcodes(void *gen, unsigned int *errorcodes, int num_codes, int *mil) {
+	struct dlopen_gen *g = (struct dlopen_gen *)gen;
+	if(NULL != g->simdl_geterrorcodes) {
+		return g->simdl_geterrorcodes(g->gen_gen, errorcodes, num_codes, mil);
+	}
+	return 0;
+}
+
+int dlopen_simgen_clearerrorcodes(void *gen) {
+	struct dlopen_gen *g = (struct dlopen_gen *)gen;
+	if(NULL != g->simdl_clearerrorcodes) {
+		return g->simdl_clearerrorcodes(g->gen_gen);
+	}
+	return 0;
+}
+
 // Declare our obdsim_generator. This is pulled in as an extern in obdsim.c
 struct obdsim_generator obdsimgen_dlopen = {
 	dlopen_simgen_name,
@@ -152,6 +193,8 @@ struct obdsim_generator obdsimgen_dlopen = {
 	dlopen_simgen_create,
 	dlopen_simgen_destroy,
 	dlopen_simgen_getvalue,
-	dlopen_simgen_idle
+	dlopen_simgen_idle,
+	dlopen_simgen_geterrorcodes,
+	dlopen_simgen_clearerrorcodes
 };
 
