@@ -80,7 +80,7 @@ int logger_simgen_create(void **gen, const char *seed) {
 	// Get the supported PIDs according to the database
 	g->supportedpids_00 = 0x01; // We can support getting higher PIDs without supporting anything else
 	g->supportedpids_20 = 0x01;
-	g->supportedpids_40 = 0x01;
+	g->supportedpids_40 = 0x00;
 	g->supportedpids_60 = 0x00;
 
 	sqlite3_stmt *pragma_stmt; // The stmt for gathering table_info
@@ -97,7 +97,7 @@ int logger_simgen_create(void **gen, const char *seed) {
 
 	while(SQLITE_DONE != sqlite3_step(pragma_stmt)) {
 		const char *columnname = sqlite3_column_text(pragma_stmt, 1);
-		if(0 == strcmp(columnname, "time")) {
+		if(0 == strcmp(columnname, "time") || 0 == strcmp(columnname, "trip")) {
 			continue;
 		}
 
@@ -229,7 +229,7 @@ int logger_simgen_getvalue(void *gen, unsigned int mode, unsigned int PID, unsig
 	const char *dbend; // ignored handle for sqlite
 	int rc = sqlite3_prepare_v2(g->db, sql, -1, &select_stmt, &dbend);
 	if(SQLITE_OK != rc) {
-		printf("Couldn't prepare select %s: %s\n", sql, sqlite3_errmsg(g->db));
+		// printf("Couldn't prepare select %s: %s\n", sql, sqlite3_errmsg(g->db));
 		sqlite3_finalize(select_stmt);
 		return 0;
 	}
@@ -256,6 +256,8 @@ struct obdsim_generator obdsimgen_logger = {
         logger_simgen_create,
         logger_simgen_destroy,
         logger_simgen_getvalue,
-		logger_simgen_idle
+	logger_simgen_idle,
+	NULL,
+	NULL
 };
 
