@@ -39,11 +39,7 @@ int random_simgen_create(void **gen, const char *seed) {
 	if(NULL != seed && '\0' != *seed) {
 		int s = atoi(seed);
 		printf("Seeding RNG with %i\n", s);
-#ifdef HAVE_SRANDOM
-		srandom(s);
-#elif defined(HAVE_SRAND)
 		srand(s);
-#endif //Random seeds
 	}
 	return 0;
 }
@@ -60,20 +56,12 @@ int random_simgen_getvalue(void *gen, unsigned int mode, unsigned int PID, unsig
 		*D = 0xFE;
 		return 4;
 	}
+	if(0x20 <= PID) return 0;
 
-#ifdef HAVE_RANDOM
-	*A = ((unsigned int) random()) & 0xFF;
-	*B = ((unsigned int) random()) & 0xFF;
-	*C = ((unsigned int) random()) & 0xFF;
-	*D = ((unsigned int) random()) & 0xFF;
-#elif defined(HAVE_RAND)
 	*A = ((unsigned int) rand()) & 0xFF;
 	*B = ((unsigned int) rand()) & 0xFF;
 	*C = ((unsigned int) rand()) & 0xFF;
 	*D = ((unsigned int) rand()) & 0xFF;
-#else
-	return -1;
-#endif //random
 
 	struct obdservicecmd *cmd = obdGetCmdForPID(PID);
 	if(NULL != cmd && 0 < cmd->bytes_returned) {
@@ -98,6 +86,8 @@ struct obdsim_generator obdsimgen_random = {
 	random_simgen_create,
 	random_simgen_destroy,
 	random_simgen_getvalue,
-	random_simgen_idle
+	random_simgen_idle,
+	NULL,
+	NULL
 };
 
