@@ -24,7 +24,6 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <errno.h>
 #include <getopt.h>
-#include <ctype.h>
 
 #include "obdconfig.h"
 #include "obdservicecommands.h"
@@ -32,10 +31,12 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #include "obdsim.h"
 #include "simport.h"
 #include "datasource.h"
+#include "mainloop.h"
 
 #ifdef OBDPLATFORM_POSIX
 #include <unistd.h>
-#include <sys/time.h>
+#include <sys/types.h>
+#include <fcntl.h>
 
 #include "posixsimport.h"
 #endif //OBDPLATFORM_POSIX
@@ -46,6 +47,10 @@ along with obdgpslogger.  If not, see <http://www.gnu.org/licenses/>.
 #include "windowssimport.h"
 #endif //OBDPLATFORM_WINDOWS
 
+#ifdef HAVE_BLUETOOTH
+#include "bluetoothsimport.h"
+#endif //HAVE_BLUETOOTH
+
 
 // Adding your plugin involves two edits here.
 // First, add an extern like the others
@@ -75,6 +80,9 @@ extern struct obdsim_generator obdsimgen_gui_fltk;
 #ifdef OBDSIMGEN_SOCKET
 extern struct obdsim_generator obdsimgen_socket;
 #endif //OBDSIMGEN_SOCKET
+#ifdef OBDSIMGEN_ERROR
+extern struct obdsim_generator obdsimgen_error;
+#endif //OBDSIMGEN_ERROR
 
 /// A list of all available generators in this build
 static struct obdsim_generator *available_generators[] = {
@@ -100,29 +108,103 @@ static struct obdsim_generator *available_generators[] = {
 	&obdsimgen_socket,
 #endif //OBDSIMGEN_SOCKET
 #ifdef OBDSIMGEN_GUI_FLTK
-	&obdsimgen_gui_fltk
+	&obdsimgen_gui_fltk,
 #endif //OBDSIMGEN_GUI_FLTK
+#ifdef OBDSIMGEN_ERROR
+	&obdsimgen_error
+#endif //OBDSIMGEN_ERROR
 };
 
-/// Default sim generator
-#define DEFAULT_SIMGEN "Cycle"
+/// Default sim generator for windows users is the gui
+#if defined(OBDSIMGEN_GUI_FLTK) && defined(OBDPLATFORM_WINDOWS)
+#undef DEFAULT_SIMGEN
+#define DEFAULT_SIMGEN "gui_fltk"
+#endif
 
-/// Default windows port
-#define DEFAULT_WINPORT "CNCA0"
 
-/// Length of time to sleep between nonblocking reads [us]
-#define OBDSIM_SLEEPTIME 10000
+/// Initialise the variables in an ECU
+void obdsim_initialiseecu(struct obdgen_ecu *e) {
+	e->simgen = NULL;
+	e->ecu_num = 0;
+	e->seed = NULL;
+	e->lasterrorcount = 0;
+	e->ffcount = 0;
+	e->dg = 0;
+	e->customdelay = 0;
+	memset(e->ff, 0, sizeof(e->ff));
+}
 
-/// It's a main loop.
-/** \param sp the simport handle
-    \param dg the data generator's void *
-    \param elm_version claim to be one of these on reset
-    \param elm_device claim to be one of these on AT@1
-    \param simgen the obdsim_generator the user has selected
-*/
-void main_loop(OBDSimPort *sp, void *dg,
-	const char *elm_version, const char *elm_device,
-	struct obdsim_generator *simgen);
+/// Initialse all variables in a simsettings
+void obdsim_initialisesimsettings(struct simsettings *s) {
+	s->e_autoprotocol = 1;
+	set_obdprotocol(OBDSIM_DEFAULT_PROTOCOLNUM, s);
+
+	s->benchmark = OBDSIM_BENCHMARKTIME;
+	s->e_currentvoltage = OBDSIM_BATTERYV;
+
+	s->device_identifier = strdup("ChunkyKs");
+	s->elm_device = strdup(OBDSIM_ELM_DEVICE_STRING);
+	s->elm_version = strdup(OBDSIM_ELM_VERSION_STRING);
+
+	s->ecu_count = 0;
+	int i;
+	for(i = 0; i<OBDSIM_MAXECUS; i++) {
+		obdsim_initialiseecu(&s->ecus[i]);
+		s->ecudelays[i].ecu = NULL;
+		s->ecudelays[i].delay = 0;
+	}
+
+	obdsim_elmreset(s);
+}
+
+/// Do an elm reset [ATZ or similar]
+void obdsim_elmreset(struct simsettings *s) {
+	s->e_headers = ELM_HEADERS;
+	s->e_spaces = ELM_SPACES;
+	s->e_echo = ELM_ECHO;
+	s->e_linefeed = ELM_LINEFEED;
+	s->e_timeout = ELM_TIMEOUT;
+	s->e_adaptive = ELM_ADAPTIVETIMING;
+	s->e_dlc = ELM_DISPLAYDLC;
+}
+
+/// Create a sorted list for the ECUs to respond in, based on their delays
+void ecudelay_order(struct simsettings *ss) {
+	int i, j;
+	for(i=0;i<ss->ecu_count;i++) {
+		ss->ecudelays[i].ecu = &ss->ecus[i];
+		ss->ecudelays[i].delay = ss->ecus[i].customdelay;
+	}
+
+	/* OH THE HORROR. BUBBLESORT [on the plus side, you only do it
+	     once, and you're only sorting six items]. Plus I haven't
+	     implemented a bubble sort in years and kinda felt like doing it. */
+	for(i=0;i<ss->ecu_count;i++) {
+		for(j=i+1;j<ss->ecu_count;j++) {
+			if(ss->ecudelays[i].delay > ss->ecudelays[j].delay) {
+				struct obdgen_ecudelays tmp;
+				memcpy(&tmp,&ss->ecudelays[i],sizeof(tmp));
+				memcpy(&ss->ecudelays[i],&ss->ecudelays[j],sizeof(tmp));
+				memcpy(&ss->ecudelays[j],&tmp,sizeof(tmp));
+			}
+		}
+	}
+
+	/* for(i=0;i<ss->ecu_count;i++) {
+		printf("ecudelay %i, num %i, delay %i\n", i,
+			ss->ecudelays[i].ecu->ecu_num, ss->ecudelays[i].delay);
+	} */
+
+	// Now go through and convert all the delays to delta since the last one
+	for(i=ss->ecu_count-1;i>0;i--) {
+		ss->ecudelays[i].delay -= ss->ecudelays[i-1].delay;
+	}
+
+	/* for(i=0;i<ss->ecu_count;i++) {
+		printf("ecudelay delta %i, num %i, delay %i\n", i,
+			ss->ecudelays[i].ecu->ecu_num, ss->ecudelays[i].delay);
+	} */
+}
 
 #ifdef OBDPLATFORM_POSIX
 /// Launch obdgpslogger connected to the pty
@@ -151,28 +233,31 @@ int main(int argc, char **argv) {
 
 	// Whether to launch screen attached to this sim
 	int launch_screen = 0;
+
+	// If you should open a real device instead of a pty
+	char *tty_device = NULL;
 #endif //OBDPLATFORM_POSIX
 
-	// Choice of generator
-	char *gen_choice = NULL;
+#ifdef HAVE_BLUETOOTH
+	// Set if they wanted a bluetooth connection
+	int bluetooth_requested = 0;
+#endif //HAVE_BLUETOOTH
 
-	// The sim generator
-	struct obdsim_generator *sim_gen;
+	// Store all settings in here
+	struct simsettings ss;
+	obdsim_initialisesimsettings(&ss);
 
 	// Logfilen name
 	char *logfile_name = NULL;
 	
-	// Pretend to be this on ATZ
-	char *elm_version = strdup(ELM_VERSION_STRING);
-
-	// Pretend to be this on AT@1
-	char *elm_device = strdup(ELM_DEVICE_STRING);
-
 #ifdef OBDPLATFORM_WINDOWS
 	// Windows port to open
 	char *winport = NULL;
 #endif //OBDPLATFORM_WINDOWS
 	
+	// Iterator/index into ecus
+	int current_ecu = 0;
+
 	int optc;
 	int mustexit = 0;
 	while ((optc = getopt_long (argc, argv, shortopts, longopts, NULL)) != -1) {
@@ -188,32 +273,74 @@ int main(int argc, char **argv) {
 				mustexit = 1;
 				break;
 			}
+			case 'L':
+				printobdprotocols();
+				mustexit = 1;
+				break;
+			case 'p':
+				if(0 != set_obdprotocol(optarg, &ss)) {
+					printf("Couldn't find protocol \"%s\", exiting\n", optarg);
+					mustexit = 1;
+				}
+				break;
 			case 'l':
 				printgenerator(0);
 				mustexit = 1;
 				break;
+			case 'n':
+				ss.benchmark = atoi(optarg);
+				break;
 			case 'V':
-				if(NULL != elm_version) {
-					free(elm_version);
+				if(NULL != ss.elm_version) {
+					free(ss.elm_version);
 				}
-				elm_version = strdup(optarg);
+				ss.elm_version = strdup(optarg);
 				break;
 			case 'D':
-				if(NULL != elm_device) {
-					free(elm_device);
+				if(NULL != ss.elm_device) {
+					free(ss.elm_device);
 				}
-				elm_device = strdup(optarg);
+				ss.elm_device = strdup(optarg);
 				break;
 			case 'v':
 				printversion();
 				mustexit = 1;
 				break;
-			case 's':
-				if(NULL != seedstr) {
-					fprintf(stderr, "Warning! Multiple seeds specified. Only last one will be used\n");
-					free(seedstr);
+			case 'g':
+				if(OBDSIM_MAXECUS <= current_ecu) {
+					fprintf(stderr, "Only support %i ECUs in this build\n", OBDSIM_MAXECUS);
+					mustexit = 1;
+				} else {
+					ss.ecus[current_ecu].simgen = find_generator(optarg);
+					ss.ecus[current_ecu].ecu_num = current_ecu;
+					if(NULL == ss.ecus[current_ecu].simgen) {
+						fprintf(stderr, "Couldn't find generator \"%s\"\n", optarg);
+						mustexit = 1;
+					}
+					current_ecu++;
 				}
-				seedstr = strdup(optarg);
+				break;
+			case 's':
+				if(current_ecu == 0) {
+					fprintf(stderr, "The seed must come after the generator\n");
+					mustexit=1;
+				}
+				if(NULL != ss.ecus[current_ecu-1].seed) {
+					fprintf(stderr, "Already provided a seed for generator %i\n", current_ecu);
+					mustexit=1;
+				}
+				ss.ecus[current_ecu-1].seed = strdup(optarg);
+				break;
+			case 'd':
+				if(current_ecu == 0) {
+					fprintf(stderr, "The custom delay must come after the generator\n");
+					mustexit=1;
+				}
+				if(0 != ss.ecus[current_ecu-1].customdelay) {
+					fprintf(stderr, "Already provided a delay for generator %i\n", current_ecu);
+					mustexit=1;
+				}
+				ss.ecus[current_ecu-1].customdelay = atoi(optarg);
 				break;
 			case 'q':
 				if(NULL != logfile_name) {
@@ -222,6 +349,11 @@ int main(int argc, char **argv) {
 				}
 				logfile_name = strdup(optarg);
 				break;
+#ifdef HAVE_BLUETOOTH
+			case 'b':
+				bluetooth_requested = 1;
+				break;
+#endif //HAVE_BLUETOOTH
 #ifdef OBDPLATFORM_POSIX
 			case 'o':
 				launch_logger = 1;
@@ -229,6 +361,12 @@ int main(int argc, char **argv) {
 			case 'c':
 				launch_screen = 1;
 				break;
+			case 't':
+				if(NULL != tty_device) {
+					free(tty_device);
+				}
+				tty_device = strdup(optarg);
+				break;
 #endif //OBDPLATFORM_POSIX
 #ifdef OBDPLATFORM_WINDOWS
 			case 'w':
@@ -239,24 +377,19 @@ int main(int argc, char **argv) {
 				winport = strdup(optarg);
 				break;
 #endif //OBDPLATFORM_WINDOWS
-			case 'g':
-				if(NULL != gen_choice) {
-					fprintf(stderr, "Warning! Multiple generators specified. Only last one will be used\n");
-					free(gen_choice);
-				}
-				gen_choice = strdup(optarg);
-				sim_gen = find_generator(gen_choice);
-				if(NULL == sim_gen) {
-					fprintf(stderr, "Couldn't find generator \"%s\"\n", gen_choice);
-					mustexit = 1;
-				}
-				break;
 			default:
 				mustexit = 1;
 				break;
 		}
 	}
 
+	ss.ecu_count = current_ecu;
+
+	if(NULL == ss.e_protocol) {
+		fprintf(stderr, "Couldn't find initial protocol %s\n", OBDSIM_DEFAULT_PROTOCOLNUM);
+		return 1;
+	}
+
 #ifdef OBDPLATFORM_POSIX
 	if(launch_logger && launch_screen) {
 		fprintf(stderr, "Error: Cannot attach both screen and logger to same sim session\n");
@@ -264,38 +397,60 @@ int main(int argc, char **argv) {
 	}
 #endif // OBDPLATFORM_POSIX
 
-	if(NULL == gen_choice) {
-		gen_choice = strdup(DEFAULT_SIMGEN);
-		sim_gen = find_generator(gen_choice);
-		if(NULL == sim_gen) {
-			fprintf(stderr, "Couldn't find default generator \"%s\"\n", gen_choice);
+	if(0 == ss.ecu_count) {
+		ss.ecus[0].ecu_num = 0;
+		ss.ecus[0].simgen = find_generator(DEFAULT_SIMGEN);
+		if(NULL == ss.ecus[0].simgen) {
+			fprintf(stderr, "Couldn't find default generator \"%s\"\n", DEFAULT_SIMGEN);
 			mustexit = 1;
 		}
+		ss.ecu_count++;
 	}
 
 	if(mustexit) return 0;
 
-	void *dg;
-
-	if(0 != sim_gen->create(&dg, seedstr)) {
-		fprintf(stderr,"Couldn't initialise data generator \"%s\"\n", gen_choice);
+	
+	int initialisation_errors = 0;
+	int i;
+	for(i=0;i<ss.ecu_count;i++) {
+		if(0 != ss.ecus[i].simgen->create(&ss.ecus[i].dg, ss.ecus[i].seed)) {
+			fprintf(stderr,"Couldn't initialise generator \"%s\" using seed \"%s\"\n",
+							ss.ecus[i].simgen->name(), ss.ecus[i].seed);
+			initialisation_errors++;
+		}
+	}
+	if(initialisation_errors > 0) {
 		return 1;
 	}
 
+	/* Getting here means all the ECUs are up and running. Now alter
+	    the order they're queried in based on customdelay */
+	ecudelay_order(&ss);
+
 	// The sim port
 	OBDSimPort *sp = NULL;
 
+#ifdef HAVE_BLUETOOTH
+	if(bluetooth_requested) {
+		sp = new BluetoothSimPort();
+	} else {
+#endif //HAVE_BLUETOOTH
+
 #ifdef OBDPLATFORM_POSIX
-	sp = new PosixSimPort();
+		sp = new PosixSimPort(tty_device);
 #endif //OBDPLATFORM_POSIX
 
 #ifdef OBDPLATFORM_WINDOWS
-	if(NULL == winport) {
-		winport = strdup(DEFAULT_WINPORT);
-	}
-	sp = new WindowsSimPort(winport);
+		if(NULL == winport) {
+			winport = strdup(DEFAULT_WINPORT);
+		}
+		sp = new WindowsSimPort(winport);
 #endif //OBDPLATFORM_WINDOWS
 
+#ifdef HAVE_BLUETOOTH
+	}
+#endif //HAVE_BLUETOOTH
+
 	if(NULL == sp || !sp->isUsable()) {
 		fprintf(stderr,"Error creating virtual port\n");
 		return 1;
@@ -324,9 +479,11 @@ int main(int argc, char **argv) {
 #endif //OBDPLATFORM_POSIX
 
 	printf("Successfully initialised obdsim, entering main loop\n");
-	main_loop(sp, dg, elm_version, elm_device, sim_gen);
+	main_loop(sp, &ss);
 
-	sim_gen->destroy(dg);
+	for(i=0;i<ss.ecu_count;i++) {
+		ss.ecus[i].simgen->destroy(ss.ecus[i].dg);
+	}
 
 	delete sp;
 
@@ -338,12 +495,26 @@ int main(int argc, char **argv) {
 		free(seedstr);
 	}
 
+	if(NULL != ss.elm_version) {
+		free(ss.elm_version);
+	}
+
+	if(NULL != ss.elm_device) {
+		free(ss.elm_device);
+	}
+
 #ifdef OBDPLATFORM_WINDOWS
 	if(NULL != winport) {
 		free(winport);
 	}
 #endif //OBDPLATFORM_WINDOWS
 
+#ifdef OBDPLATFORM_POSIX
+	if(NULL != tty_device) {
+		free(tty_device);
+	}
+#endif //OBDPLATFORM_POSIX
+
 	return 0;
 }
 
@@ -362,10 +533,13 @@ int spawnlogger(char *ptyname) {
 
 	// In child
 	execlp("obdgpslogger", "obdgpslogger",
+		"--baud", "-1",                      // Don't modify baudrate
+		"--modifybaud", "-1",                // Don't upgrade baudrate
 		"--serial", ptyname,                 // Connect to this pty
 		"--db", "./obdgpsloggertmp.db",      // Dump to this database
 		"--serial-log", "./serialcomms.txt", // Log serial comms to this file
 		"--spam-stdout",                     // Spam stdout
+		"--enable-optimisations",            // Enable elm optimisations
 		NULL);
 	perror("Couldn't exec obdgpslogger");
 	exit(0);
@@ -380,6 +554,13 @@ int spawnscreen(char *ptyname) {
 	}
 
 	if(0 < pid) {
+		// To avoid seeing console spam in screen, dump stdout
+		int fd;
+		close(STDOUT_FILENO);
+		if ((fd = open("/dev/null", O_RDWR, 0)) != -1) {
+			dup2(fd, STDOUT_FILENO);
+		}
+
 		return 0;
 	}
 
@@ -392,268 +573,6 @@ int spawnscreen(char *ptyname) {
 }
 #endif //OBDPLATFORM_POSIX
 
-void main_loop(OBDSimPort *sp, void *dg,
-		const char *elm_version, const char *elm_device,
-	 	struct obdsim_generator *simgen) {
-
-	char *line; // Single line from the other end of the device
-	char previousline[1024]; // Blank lines mean re-run previous command
-
-	// Elm327 options go here.
-	int e_headers = ELM_HEADERS; // Whether to show headers
-	int e_spaces = ELM_SPACES; // Whether to show spaces
-	int e_echo = ELM_ECHO; // Whether to echo commands
-	char *device_identifier = strdup("ChunkyKs");
-
-	sp->setEcho(e_echo);
-
-	int mustexit = 0;
-	while(!mustexit) {
-		// Begin main loop by idling for OBDSIM_SLEEPTIME ms
-		struct timeval starttime; // start time through loop
-		struct timeval endtime; // end time through loop
-		struct timeval selecttime; // =endtime-starttime [for select()]
-		if(0 != gettimeofday(&starttime,NULL)) {
-			perror("Couldn't gettimeofday for sim mainloop starttime");
-			break;
-		}
-
-		if(NULL != simgen->idle) {
-			if(0 != simgen->idle(dg,OBDSIM_SLEEPTIME/1000)) {
-				mustexit = 1;
-				break;
-			}
-		}
-
-		if(0 != gettimeofday(&endtime,NULL)) {
-			perror("Couldn't gettimeofday for sim mainloop endtime");
-			break;
-		}
-
-		selecttime.tv_sec = endtime.tv_sec - starttime.tv_sec;
-		if (selecttime.tv_sec != 0) {
-				endtime.tv_usec += 1000000*selecttime.tv_sec;
-				selecttime.tv_sec = 0;
-		}
-		selecttime.tv_usec = (OBDSIM_SLEEPTIME) - (endtime.tv_usec - starttime.tv_usec);
-		if(selecttime.tv_usec > 0) {
-			select(0,NULL,NULL,NULL,&selecttime);
-		}
-
-
-		// Now the actual choise-response thing
-		line = sp->readLine(); // This is the input line
-		char response[1024]; // This is the response
-
-		if(NULL == line) continue;
-		if(0 == strlen(line)) {
-			line = previousline;
-		} else {
-			strncpy(previousline, line, sizeof(previousline));
-		}
-
-		int i;
-		for(i=strlen(line)-1;i>=0;i--) { // Strlen is expensive, kids.
-			line[i] = toupper(line[i]);
-		}
-
-		// printf("obdsim got request: %s\n", line);
-
-		if(NULL != strstr(line, "EXIT")) {
-			printf("Received EXIT via serial port. Sim Exiting\n");
-			mustexit=1;
-			continue;
-		}
-
-		// If we recognised the command
-		int command_recognised = 0;
-
-		if('A' == line[0] && 'T' == line[1]) {
-			// This is an AT command
-			int atopt_i; // If they pass an integer option
-
-			char *at_cmd = line + 2;
-
-			for(; ' ' == *at_cmd; at_cmd++) { // Find the first non-space character in the AT command
-			}
-
-			if(1 == sscanf(at_cmd, "H%i", &atopt_i)) {
-				printf("Headers %s\n", atopt_i?"enabled":"disabled");
-				e_headers = atopt_i;
-				command_recognised = 1;
-				snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-			}
-
-			if(1 == sscanf(at_cmd, "L%i", &atopt_i)) {
-				printf("Linefeed %s\n", atopt_i?"enabled":"disabled");
-				command_recognised = 1;
-				snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-			}
-
-			if(1 == sscanf(at_cmd, "S%i", &atopt_i)) {
-				printf("Spaces %s\n", atopt_i?"enabled":"disabled");
-				e_spaces = atopt_i;
-				command_recognised = 1;
-				snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-			}
-
-			if(1 == sscanf(at_cmd, "E%i", &atopt_i)) {
-				printf("Echo %s\n", atopt_i?"enabled":"disabled");
-				e_echo = atopt_i;
-				sp->setEcho(e_echo);
-				snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-				command_recognised = 1;
-			}
-
-			if(1 == sscanf(at_cmd, "@%i", &atopt_i)) {
-				if(1 == atopt_i) {
-					snprintf(response, sizeof(response), ELM_NEWLINE "%s" ELM_PROMPT, elm_device);
-					command_recognised = 1;
-				} else if(2 == atopt_i) {
-					snprintf(response, sizeof(response), ELM_NEWLINE "%s" ELM_PROMPT, device_identifier);
-					command_recognised = 1;
-				} else if(3 == atopt_i) {
-					snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-					free(device_identifier);
-					char *newid = at_cmd+2;
-					while(' ' == *newid) newid++;
-					device_identifier = strdup(newid);
-					printf("Set device identifier to \"%s\"\n", device_identifier);
-					command_recognised = 1;
-				}
-			}
-
-			if(1 == sscanf(at_cmd, "ST%i", &atopt_i)) {
-				snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-				command_recognised = 1;
-			}
-
-			if(0 == strncmp(at_cmd, "RV", 2)) {
-				snprintf(response, sizeof(response), ELM_NEWLINE "%s" ELM_PROMPT, "11.8");
-				command_recognised = 1;
-			}
-
-			if(0 == strncmp(at_cmd, "DPN", 3)) {
-				snprintf(response, sizeof(response), ELM_NEWLINE "%s" ELM_PROMPT, ELM_PROTOCOL_NUMBER);
-				command_recognised = 1;
-			} else if(0 == strncmp(at_cmd, "DP", 2)) {
-				snprintf(response, sizeof(response), ELM_NEWLINE "%s" ELM_PROMPT, ELM_PROTOCOL_DESCRIPTION);
-				command_recognised = 1;
-			} else if('D' == at_cmd[0]) {
-				printf("Defaults\n");
-
-				e_headers = ELM_HEADERS;
-				e_spaces = ELM_SPACES;
-				e_echo = ELM_ECHO;
-				sp->setEcho(e_echo);
-
-				snprintf(response, sizeof(response), "%s", ELM_OK_PROMPT);
-				command_recognised = 1;
-			}
-
-			if('Z' == at_cmd[0] || 0 == strncmp(at_cmd, "WS", 2)) {
-				if('Z' == at_cmd[0]) {
-					printf("Reset\n");
-				} else {
-					printf("Warm Start\n");
-				}
-
-				e_headers = ELM_HEADERS;
-				e_spaces = ELM_SPACES;
-				e_echo = ELM_ECHO;
-				sp->setEcho(e_echo);
-
-				command_recognised = 1;
-				snprintf(response, sizeof(response), ELM_NEWLINE "%s" ELM_PROMPT, elm_version);
-			}
-
-
-			if(0 == command_recognised) {
-				snprintf(response, sizeof(response), "%s", ELM_QUERY_PROMPT);
-			}
-
-			sp->writeData(response);
-
-			continue;
-		}
-
-
-		int num_vals_read; // Number of values parsed from the sscanf line
-		int vals[3]; // Read up to three vals
-		num_vals_read = sscanf(line, "%02x %02x %1x", &vals[0], &vals[1], &vals[2]);
-
-		if(num_vals_read == 0) {
-				snprintf(response, sizeof(response), "%s", ELM_QUERY_PROMPT);
-		} else if(num_vals_read == 1) {
-				if(0x03 == vals[0] || 0x04 == vals[0]) {
-					// TODO: Error code handling
-					snprintf(response, sizeof(response), ELM_NODATA_PROMPT);
-				} else {
-					snprintf(response, sizeof(response), "%s", ELM_QUERY_PROMPT);
-				}
-		} else { // Num_vals_read == 2 or 3
-
-			struct obdservicecmd *cmd = obdGetCmdForPID(vals[1]);
-			if(NULL == cmd) {
-				snprintf(response, sizeof(response), "%s", ELM_QUERY_PROMPT);
-			} else {
-
-				// Here's the meat & potatoes of the whole application
-
-				if(0x02 == vals[0] || 0x09 == vals[0]) {
-					snprintf(response, sizeof(response), "%s", ELM_NODATA_PROMPT);
-					sp->writeData(response);
-					continue;
-				}
-
-				// Success!
-				unsigned int abcd[4];
-				int count = simgen->getvalue(dg, vals[0], vals[1],
-								abcd+0, abcd+1, abcd+2, abcd+3);
-
-				// printf("Returning %i values for %02X %02X\n", count, vals[0], vals[1]);
-
-				if(-1 == count) {
-					mustexit = 1;
-					break;
-				}
-
-				if(0 == count) {
-					snprintf(response, sizeof(response), "%s", ELM_NODATA_PROMPT);
-					sp->writeData(response);
-					continue;
-				}
-
-				char header[16];
-				if(e_headers) {
-					snprintf(header, sizeof(header), "%s%s%s%s",
-						"7E8", e_spaces?" ":"",
-						"06", e_spaces?" ":"");
-				} else {
-					snprintf(header, sizeof(header), "");
-				}
-				int i;
-				snprintf(response, sizeof(response), ELM_NEWLINE "%s%02X%s%02X",
-							header,
-							vals[0]+0x40, e_spaces?" ":"", vals[1]);
-				for(i=0;i<count;i++) {
-					char shortbuf[10];
-					snprintf(shortbuf, sizeof(shortbuf), "%s%02X",
-							e_spaces?" ":"", abcd[i]);
-					// printf("shortbuf: '%s'   i: %i\n", shortbuf, abcd[i]);
-					strcat(response, shortbuf);
-				}
-				strcat(response, ELM_PROMPT);
-			}
-		}
-
-		sp->writeData(response);
-	}
-
-	free(device_identifier);
-
-}
-
 void show_genhelp(struct obdsim_generator *gen) {
 	if(NULL == gen) {
 		fprintf(stderr, "Cannot print help for nonexistent generator\n");
@@ -667,7 +586,7 @@ void show_genhelp(struct obdsim_generator *gen) {
 }
 
 static struct obdsim_generator *find_generator(const char *gen_name) {
-	int i;
+	size_t i;
 	if(NULL == gen_name) return NULL;
 
 	for(i=0; i<sizeof(available_generators)/sizeof(available_generators[0]); i++) {
@@ -678,22 +597,78 @@ static struct obdsim_generator *find_generator(const char *gen_name) {
 	return NULL;
 }
 
+int set_obdprotocol(const char *prot, struct simsettings *ss) {
+	if(NULL == prot) {
+		return -1;
+	}
+
+	// Allow spaces in various places even though behaviour isn't very well defined
+	const char *p = prot;
+	while(' ' == *p && '\0' != *p) p++;
+	if('\0' == *p) {
+		return -1;
+	}
+
+	int e_autoprotocol = 0;
+	if('A' == *p || 'a' == *p) {
+		e_autoprotocol = 1;
+		p++;
+	}
+	while(' ' == *p && '\0' != *p) p++;
+
+	struct obdiiprotocol *e_protocol = find_obdprotocol(p);
+	p++;
+	if(NULL == e_protocol) {
+		return -1;
+	}
+	while(' ' == *p && '\0' != *p) p++;
+
+	if('A' == *p || 'a' == *p) {
+		e_autoprotocol = 1;
+	}
+
+	ss->e_protocol = e_protocol;
+	ss->e_autoprotocol = e_autoprotocol;
+
+	return 0;
+}
+
+struct obdiiprotocol *find_obdprotocol(const char *protocol_num) {
+	int i;
+	for(i=0; i<sizeof(obdprotocols)/sizeof(obdprotocols[0]); i++) {
+		if(*protocol_num == obdprotocols[i].protocol_num) {
+			return &(obdprotocols[i]);
+		}
+	}
+	return NULL;
+}
+
+
 void printhelp(const char *argv0) {
 	printf("Usage: %s [params]\n"
-		"   [-s|--seed=<generator-specific-string>]\n"
-		"   [-g|--generator=<name of generator>]\n"
+		"   [-g|--generator=<name of generator>\n"
+		"       [-s|--seed=<generator-seed>]\n"
+		"       [-d|--customdelay=<ecu delay(ms)>]\n"
+		"     ]\n"
 		"   [-q|--logfile=<logfilename to write to>]\n"
 		"   [-V|--elm-version=<pretend to be this on ATZ>]\n"
 		"   [-D|--elm-device=<pretend to be this on AT@1>]\n"
+		"   [-L|--list-protocols]\n"
+		"   [-p|--protocol=<OBDII protocol>]\n"
 #ifdef OBDPLATFORM_POSIX
 		"   [-o|--launch-logger]\n"
-		"   [-c|--launch-screen] [use ctrl-a,k to exit screen]\n"
+		"   [-c|--launch-screen] [\"EXIT\" or C-a,k to exit]\n"
+		"   [-t|--tty-device=<real /dev/ entry to open>]\n"
 #endif //OBDPLATFORM_POSIX
 #ifdef OBDPLATFORM_WINDOWS
 		"   [-w|--com-port=<windows COM port>]\n"
 #endif //OBDPLATFORM_WINDOWS
+#ifdef HAVE_BLUETOOTH
+		"   [-b|--bluetooth]\n"
+#endif //HAVE_BLUETOOTH
 		"   [-e|--genhelp=<name of generator>]\n"
 		"   [-l|--list-generators]\n"
+		"   [-n|--benchmark=<seconds>]\n"
 		"   [-v|--version] [-h|--help]\n", argv0);
 }
 
@@ -701,6 +676,13 @@ void printversion() {
 	printf("Version: %i.%i\n", OBDGPSLOGGER_MAJOR_VERSION, OBDGPSLOGGER_MINOR_VERSION);
 }
 
+void printobdprotocols() {
+	int i;
+	for(i=0; i<sizeof(obdprotocols)/sizeof(obdprotocols[0]); i++) {
+		printf("%c  %s\n", obdprotocols[i].protocol_num,  obdprotocols[i].protocol_desc);
+	}
+}
+
 void printgenerator(int verbose) {
 	if(verbose) {
 		printf("The generators built into this sim:\n");
@@ -709,7 +691,7 @@ void printgenerator(int verbose) {
 	// If we find the one currently #defined as default
 	int found_default = 0;
 
-	int i;
+	size_t i;
 	for(i=0; i<sizeof(available_generators)/sizeof(available_generators[0]); i++) {
 		int is_default = !strcmp(DEFAULT_SIMGEN, available_generators[i]->name());
 		if(is_default) found_default = 1;
